﻿using System;
using System.Collections.Generic;
using System.Dynamic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Freebase4net.Operators;
using Freebase4net;
using System.Threading.Tasks;
using System.Diagnostics;
using System.Threading;

namespace Freebase4net.Tests
{
    [TestClass]
    public class MqlReadServiceTests
    {
        private MqlReadService _readService;

        public MqlReadServiceTests()
        {
            _readService = FreebaseServicesBuilder.CreateMqlReadService();
        }

        [TestMethod]
        public void Read_ObjectWithLikeOperator_StatusCodeOk()
        {
            dynamic thepolice = new ExpandoObject();
            thepolice.type = "/music/artist";
            thepolice.name = new LikeOperator("^The Sco*$");


            var result = _readService.Read(thepolice);

            Assert.AreEqual("/music/artist", (string)result.Results[0].type);
            Assert.AreEqual(System.Net.HttpStatusCode.OK, result.Status);
        }

        [TestMethod]
        public void Read_DictionaryOfObjects_StatusCodeOk()
        {
            dynamic thepolice = new ExpandoObject();
            thepolice.type = "/music/artist";
            thepolice.name = "The Scorpions";
            thepolice.album = new Dictionary<Object, Object>();
            thepolice.album.Add("name", null);
            thepolice.album.Add("limit", 2);
            thepolice.album.Add("genre", Freeb.CreateArrayWithEmptyObject());


            var result = _readService.Read(thepolice);


            Assert.AreEqual(System.Net.HttpStatusCode.OK, result.Status);
        }

        [TestMethod]
        public void Read_ComplexEmbeddedObject_StatusCodeOk()
        {
            dynamic director = new ExpandoObject();
            director.name = null;
            director.id = null;
            director.type = "/film/editor";
            director.film = new Dictionary<Object, Object>() {
                {"name",null},
                {"id",null},
                {"starring",new Dictionary<Object,Object>()
                    {
                        {"actor","Theresa Russell"}
                    }
                }
            };


            var result = _readService.Read(director);


            Assert.AreEqual(System.Net.HttpStatusCode.OK, result.Status);
        }

        [TestMethod]
        public void ReadMultiple_MultipleQueries_StatusCodeOk()
        {
            dynamic thescorpions = new ExpandoObject();
            thescorpions.name = "The Scorpions";
            thescorpions.type = "/music/artist";

            dynamic thepolice = new ExpandoObject();
            thepolice.name = "The Police";
            thepolice.type = "/music/artist";

            MultipleMqlReadServiceResponse multiFreebaseResponse = _readService.ReadMultiple(thescorpions, thepolice);

            foreach (var res in multiFreebaseResponse.Results)
            {
                Assert.AreEqual(System.Net.HttpStatusCode.OK, res.Status);
            }
        }
    }
}